/* LibTomCrypt, modular cryptographic library -- Tom St Denis
 *
 * LibTomCrypt is a library that provides various cryptographic
 * algorithms in a highly modular and flexible manner.
 *
 * The library is free for all purposes without any express
 * guarantee it works.
 *
 * Tom St Denis, tomstdenis@gmail.com, http://libtomcrypt.org
 *
 * slightly modified by the author of TCTEMP
 */
#include "tomcrypt.h"

/**
  @file hash_memory.c
  Hash memory helper, Tom St Denis
*/

/**
  Hash a block of memory and store the digest.
  @param hash   The index of the hash you wish to use
  @param in     The data you wish to hash
  @param inlen  The length of the data to hash (octets)
  @param out    [out] Where to store the digest
  @param outlen [in/out] Max size and resulting size of the digest
  @return CRYPT_OK if successful
*/
int tc_hash_memory(const struct ltc_hash_descriptor *pdHash, const unsigned char *in, 
  unsigned long inlen, unsigned char *out, unsigned long *outlen, hash_state *md)
{
    int err;

	LTC_ARGCHK(pdHash != NULL);
	LTC_ARGCHK(in     != NULL);
    LTC_ARGCHK(out    != NULL);
    LTC_ARGCHK(outlen != NULL);
	LTC_ARGCHK(md     != NULL);

    if (*outlen < pdHash->hashsize) {
       return CRYPT_BUFFER_OVERFLOW;
    }

    if ((err = pdHash->init(md)) != CRYPT_OK) {
       goto LBL_ERR;
    }
    if ((err = pdHash->process(md, in, inlen)) != CRYPT_OK) {
       goto LBL_ERR;
    }
    err = pdHash->done(md, out);
    *outlen = pdHash->hashsize;
LBL_ERR:

    return err;
}

/* $Source: /cvs/libtom/libtomcrypt/src/hashes/helper/hash_memory.c,v $ */
/* $Revision: 1.3 $ */
/* $Date: 2005/05/05 14:35:58 $ */
