/*
    Project: TCTEMP
    Version: V1.22
    File   : na.h

    Copyright (c) 2006-2012 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#ifndef __NA_H
#define __NA_H

#ifdef BUILD_FLAGS
#  if BUILD_FLAGS & 1
#    define DEBUG 1
#  endif
#endif
#ifndef DEBUG
#  define DEBUG 0
#endif

#ifdef __cplusplus
extern "C" {
#endif

#include "nt_api.h"

/*----------------------------------------------------------------------------*/
/* Win32 types                                                                */

typedef int              INT, *PINT;
typedef int              BOOL, *PBOOL; /* != BOOLEAN */
typedef unsigned __int8  BYTE, *PBYTE;
typedef unsigned __int16 WORD, *PWORD;
typedef unsigned __int32 DWORD, *PDWORD;

#if WINVER <= 0x0500
typedef unsigned __int8  UINT8, *PUINT8;
typedef unsigned __int16 UINT16, *PUINT16;
typedef unsigned __int32 UINT32, *PUINT32;
typedef unsigned __int64 UINT64, *PUINT64;
#endif

#define LOWORD(dw) ((WORD)((dw) & 0xffff))
#define HIWORD(dw) ((WORD)((dw) >> 16))
#define LOBYTE(w) ((BYTE)((w) & 0xff))
#define HIBYTE(w) ((BYTE)((w) >> 8))

#define MAX_PATH 260
#define MAX_NT_PATH (MAX_PATH + 4)

#define countof(arr) (sizeof(arr)/sizeof(arr[0]))

/*----------------------------------------------------------------------------*/
/* Native API types                                                           */

#define NAAPI __stdcall
#define NAAPI_CDECL _cdecl

#define NA_OFFSET(s, elem) ((size_t) &((s *)0)->elem)

/*----------------------------------------------------------------------------*/
/* Debug macros                                                               */

#ifndef NA_NO_DEBUG
   void NAAPI_CDECL NaDbgPrintfFunc(PCWSTR pcwszFormat, ...);
#  if DEBUG
#    define NaWIDEN2(x) L ## x
#    define NaWIDEN(x)  NaWIDEN2(x)
#    define __NA_WFILE__ NaWIDEN(__FILE__)
     void NAAPI NaDbgAssertionFailed(PCWSTR pcwszFile, ULONG ulLineNo);
     void NAAPI NaDbgPutsFunc(PCWSTR pcwszText);
#    define NaAssert(x) if (!(x)) NaDbgAssertionFailed(__NA_WFILE__, __LINE__);
#    define NaDbgPuts(x) NaDbgPutsFunc(x)
#    define NaDbgPrintf NaDbgPrintfFunc
#  else
#    define NaAssert(x)
#    define NaDbgPuts(x)
#    define NaDbgPrintf __noop
#  endif
#endif

/*----------------------------------------------------------------------------*/
/* Startup                                                                    */

BOOL NAAPI NaInitialize(PVOID *ppCtx, SIZE_T cbCtxSize, SIZE_T cbAlignment);
void NAAPI NaUninitialize(void);

/*----------------------------------------------------------------------------*/
/* Display                                                                    */

void NAAPI       NaPuts(PCWSTR pcwszText);
void NAAPI_CDECL NaPrintf(PCWSTR pcwszFormat, ...);

/*----------------------------------------------------------------------------*/
/* File                                                                       */

BOOL NAAPI NaCreateFile(PHANDLE phFile, PCWSTR pcwszFileName,
  ACCESS_MASK amDesiredAccess, ULONG ulFileAttributes, ULONG ulShareAccess,
  ULONG ulCreateDisposition, ULONG ulCreateOptions);
#define NaOpenFile(phFile, pcwszFileName, amDesiredAccess, ulShareAccess, \
  ulOpenOptions) \
  NaCreateFile(phFile, pcwszFileName, amDesiredAccess, 0, ulShareAccess, \
  FILE_OPEN, ulOpenOptions)
#define NaOpenDevice(phDevice, pcwszDeviceName) \
  NaOpenFile(phDevice, pcwszDeviceName, GENERIC_ALL, 0, 0)
#define NaCloseFile(hFile) NtClose(hFile)

BOOL NAAPI NaReadFile(HANDLE hFile, PVOID pBuffer, ULONG ulLength,
  PULONG pulBytesRead, PLARGE_INTEGER pliByteOffset);
BOOL NAAPI NaWriteFile(HANDLE hFile, PVOID pBuffer, ULONG ulLength,
  PULONG pulBytesWritten, PLARGE_INTEGER pliByteOffset);
BOOL NAAPI NaDeviceIoControl(HANDLE hDevice, ULONG ulControlCode,
  PVOID pInputBuffer, ULONG ulInputBufferLength, PVOID pOutputBuffer,
  ULONG ulOutputBufferLength, PULONG pulBytesReturned);

BOOL NAAPI NaQueryFileInfo(HANDLE hFile, PVOID pFileInformation,
  ULONG ulFileInformationSize, FILE_INFORMATION_CLASS fic);
BOOL NAAPI NaSetFileInfo(HANDLE hFile, PVOID pFileInformation,
  ULONG ulFileInformationSize, FILE_INFORMATION_CLASS fic);

#define NaSeek(hFile, pliByteOffset) \
  NaSetFileInfo(hFile, pliByteOffset, sizeof(LARGE_INTEGER), \
    FilePositionInformation)
BOOL NAAPI NaGetFileAttributes(HANDLE hFile, PULONG pulAttributes);
BOOL NAAPI NaGetFileLength(HANDLE hFile, PULONGLONG pullFileLength);

/*----------------------------------------------------------------------------*/
/* Filename                                                                   */

BOOL NAAPI NaGetNtFileName(PWSTR pwszDest, SIZE_T cchDest, PCWSTR pcwszSrc);
BOOL NAAPI NaGetNtDriveName(PWSTR pwszDest, DWORD cchDest, WCHAR chDriveLetter);

/*----------------------------------------------------------------------------*/
/* Miscellany                                                                 */

BOOL NAAPI NaGetSystemInformation(SYSTEM_INFORMATION_CLASS sic,
  PVOID pBuffer, ULONG cbBufferSize, PULONG pcbReturnLength);

/* relative timeout in msec */
#define NA_TIMEOUT(t) (((LONGLONG)t)*-10000)
void NAAPI NaSleep(DWORD dwTime);

#ifdef __cplusplus
}
#endif

#endif
